@@ -51,7 +51,6 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
-
 /*
  * WARNING:  Do not change the order of these fields or add additional
  * fields at the beginning of the structure.  If you do, MPI plugins
@@ -59,6 +58,8 @@
  * at the end of the structure.
  */
 typedef struct slurm_mpi_ops {
+	int          (*slurmstepd_prefork)(const slurmd_job_t *job,
+					   char ***env);
 	int          (*slurmstepd_init)   (const mpi_plugin_task_info_t *job,
 					   char ***env);
 	mpi_plugin_client_state_t *
@@ -78,6 +79,7 @@ struct slurm_mpi_context {
 
 static slurm_mpi_context_t g_context = NULL;
 static pthread_mutex_t      context_lock = PTHREAD_MUTEX_INITIALIZER;
+static bool init_run = false;
 
 static slurm_mpi_context_t
 _slurm_mpi_context_create(const char *mpi_type)
@@ -142,6 +144,7 @@ _slurm_mpi_get_ops( slurm_mpi_context_t c )
 	 * declared for slurm_mpi_ops_t.
 	 */
 	static const char *syms[] = {
+		"p_mpi_hook_slurmstepd_prefork",
 		"p_mpi_hook_slurmstepd_task",
 		"p_mpi_hook_client_prelaunch",
 		"p_mpi_hook_client_single_task_per_node",
@@ -209,6 +212,9 @@ int _mpi_init (char *mpi_type)
 	char *full_type = NULL;
 	int got_default = 0;
 
+	if ( init_run && g_context )
+		return retval;
+
 	slurm_mutex_lock( &context_lock );
 
 	if ( g_context )
@@ -242,7 +248,7 @@ int _mpi_init (char *mpi_type)
 		g_context = NULL;
 		retval = SLURM_ERROR;
 	}
-
+	init_run = true;
 
 done:
 	if(got_default)
@@ -265,6 +271,14 @@ int mpi_hook_slurmstepd_init (char ***env)
 	return SLURM_SUCCESS;
 }
 
+int mpi_hook_slurmstepd_prefork (const slurmd_job_t *job, char ***env)
+{
+	if (mpi_hook_slurmstepd_init(env) == SLURM_ERROR)
+		return SLURM_ERROR;
+
+	return (*(g_context->ops.slurmstepd_prefork))(job, env);
+}
+
 int mpi_hook_slurmstepd_task (const mpi_plugin_task_info_t *job, char ***env)
 {
 	if (mpi_hook_slurmstepd_init(env) == SLURM_ERROR)
@@ -322,6 +336,7 @@ int mpi_fini (void)
 	if (!g_context)
 		return SLURM_SUCCESS;
 
+	init_run = false;
 	rc = _slurm_mpi_context_destroy(g_context);
 	return rc;
 }
