@@ -52,8 +52,10 @@ uint16_t _get_slurm_version(uint32_t rpc_version)
 {
 	uint16_t version;
 
-	if (rpc_version >= 9)
+	if (rpc_version >= 10)
 		version = SLURM_PROTOCOL_VERSION;
+	else if (rpc_version >= 9)
+		version = SLURM_2_3_PROTOCOL_VERSION;
 	else if (rpc_version >= 8)
 		version = SLURM_2_2_PROTOCOL_VERSION;
 	else if (rpc_version >= 6)
@@ -82,6 +84,7 @@ int check_header_version(header_t * header)
 
 	if (slurmdbd_conf) {
 		if ((header->version != SLURM_PROTOCOL_VERSION)     &&
+		    (header->version != SLURM_2_3_PROTOCOL_VERSION) &&
 		    (header->version != SLURM_2_2_PROTOCOL_VERSION) &&
 		    (header->version != SLURM_2_1_PROTOCOL_VERSION))
 			slurm_seterrno_ret(SLURM_PROTOCOL_VERSION_ERROR);
@@ -116,6 +119,7 @@ int check_header_version(header_t * header)
 		case REQUEST_PARTITION_INFO:
 		case REQUEST_PING:
 		case REQUEST_PRIORITY_FACTORS:
+		case REQUEST_REBOOT_NODES:
 		case REQUEST_RECONFIGURE:
 		case REQUEST_RESERVATION_INFO:
 		case REQUEST_SET_DEBUG_FLAGS:
@@ -143,10 +147,12 @@ int check_header_version(header_t * header)
 		case REQUEST_UPDATE_NODE:
 		case REQUEST_UPDATE_PARTITION:
 		case REQUEST_UPDATE_RESERVATION:
-			if ((header->version == SLURM_2_2_PROTOCOL_VERSION)
+			if ((header->version == SLURM_2_3_PROTOCOL_VERSION)
+			    || (header->version == SLURM_2_2_PROTOCOL_VERSION)
 			    || (header->version == SLURM_2_1_PROTOCOL_VERSION))
 				break;
 		default:
+			debug("unsupported RPC %d", header->msg_type);
 			slurm_seterrno_ret(SLURM_PROTOCOL_VERSION_ERROR);
 			break;
 		}
@@ -170,15 +176,17 @@ void init_header(header_t *header, slurm_msg_t *msg, uint16_t flags)
 	if (msg->protocol_version != (uint16_t)NO_VAL)
 		header->version = msg->protocol_version;
 	else if (working_cluster_rec)
-		header->version = _get_slurm_version(
+		msg->protocol_version = header->version = _get_slurm_version(
 			working_cluster_rec->rpc_version);
 	else if ((msg->msg_type == ACCOUNTING_UPDATE_MSG) ||
 	         (msg->msg_type == ACCOUNTING_FIRST_REG)) {
 		uint32_t rpc_version =
 			((accounting_update_msg_t *)msg->data)->rpc_version;
-		header->version = _get_slurm_version(rpc_version);
+		msg->protocol_version = header->version =
+			_get_slurm_version(rpc_version);
 	} else
-		header->version = SLURM_PROTOCOL_VERSION;
+		msg->protocol_version = header->version =
+			SLURM_PROTOCOL_VERSION;
 
 	header->flags = flags;
 	header->msg_type = msg->msg_type;
