@@ -283,7 +283,7 @@ extern slurm_step_layout_t *slurm_step_layout_copy(
 extern void pack_slurm_step_layout(slurm_step_layout_t *step_layout,
 				   Buf buffer, uint16_t protocol_version)
 {
-	uint16_t i = 0;
+	uint32_t i = 0;
 
 	if (protocol_version >= SLURM_2_3_PROTOCOL_VERSION) {
 		if (step_layout)
@@ -650,15 +650,14 @@ static int _task_layout_cyclic(slurm_step_layout_t *step_layout,
 	int i, j, taskid = 0;
 	bool over_subscribe = false;
 
-	for (i=0; i<step_layout->node_cnt; i++) {
-		step_layout->tids[i] = xmalloc(sizeof(uint32_t)
-					       * step_layout->task_cnt);
-	}
 	for (j=0; taskid<step_layout->task_cnt; j++) {   /* cycle counter */
 		bool space_remaining = false;
 		for (i=0; ((i<step_layout->node_cnt)
 			   && (taskid<step_layout->task_cnt)); i++) {
 			if ((j<cpus[i]) || over_subscribe) {
+				xrealloc(step_layout->tids[i], sizeof(uint32_t)
+					 * (step_layout->tasks[i] + 1));
+
 				step_layout->tids[i][step_layout->tasks[i]] =
 					taskid;
 				taskid++;
