@@ -513,12 +513,13 @@ no_rollup_change:
 
 		xstrfmtcat(query, "time_start=%ld, job_name='%s', state=%u, "
 			   "cpus_alloc=%u, nodes_alloc=%u, id_qos=%u, "
-			   "id_assoc=%u, id_wckey=%u, id_resv=%u, timelimit=%u "
+			   "id_assoc=%u, id_wckey=%u, id_resv=%u, "
+			   "timelimit=%u, time_eligible=%ld "
 			   "where job_db_inx=%d",
 			   start_time, jname, job_state,
 			   job_ptr->total_cpus, node_cnt, job_ptr->qos_id,
 			   job_ptr->assoc_id, wckeyid,
-			   job_ptr->resv_id, job_ptr->time_limit,
+			   job_ptr->resv_id, job_ptr->time_limit, begin_time,
 			   job_ptr->db_index);
 		debug3("%d(%s:%d) query\n%s",
 		       mysql_conn->conn, THIS_FILE, __LINE__, query);
@@ -916,7 +917,6 @@ extern int as_mysql_step_complete(mysql_conn_t *mysql_conn,
 				  struct step_record *step_ptr)
 {
 	time_t now;
-	int elapsed;
 	int comp_status;
 	int cpus = 0;
 	struct jobacctinfo *jobacct = (struct jobacctinfo *)step_ptr->jobacct;
@@ -926,7 +926,7 @@ extern int as_mysql_step_complete(mysql_conn_t *mysql_conn,
 	char *query = NULL;
 	int rc =SLURM_SUCCESS;
 	uint32_t exit_code = 0;
-	time_t start_time, submit_time;
+	time_t submit_time;
 
 	if (!step_ptr->job_ptr->db_index
 	    && ((!step_ptr->job_ptr->details
@@ -937,14 +937,10 @@ extern int as_mysql_step_complete(mysql_conn_t *mysql_conn,
 		return SLURM_ERROR;
 	}
 
-	if (step_ptr->job_ptr->resize_time) {
-		submit_time = start_time = step_ptr->job_ptr->resize_time;
-		if (step_ptr->start_time > submit_time)
-			start_time = step_ptr->start_time;
-	} else {
-		start_time = step_ptr->start_time;
+	if (step_ptr->job_ptr->resize_time)
+		submit_time = step_ptr->job_ptr->resize_time;
+	else
 		submit_time = step_ptr->job_ptr->details->submit_time;
-	}
 
 	if (jobacct == NULL) {
 		/* JobAcctGather=slurmdb_gather/none, no data to process */
@@ -974,9 +970,6 @@ extern int as_mysql_step_complete(mysql_conn_t *mysql_conn,
 #endif
 	}
 
-	if ((elapsed = (now - start_time)) < 0)
-		elapsed = 0;	/* For *very* short jobs, if clock is wrong */
-
 	exit_code = step_ptr->exit_code;
 	if (WIFSIGNALED(exit_code)) {
 		comp_status = JOB_CANCELLED;
