@@ -242,8 +242,7 @@ static void _state_time_string(char **extra, uint32_t state,
 		}
 		break;
 	case JOB_SUSPENDED:
-		/* FIX ME: this should do something with the suspended
-		   table, but it doesn't right now. */
+		/* Handle this the same way we handle RUNNING. */
 	case JOB_RUNNING:
 		if (start) {
 			if (!end) {
@@ -252,7 +251,7 @@ static void _state_time_string(char **extra, uint32_t state,
 					   "((!t1.time_end && t1.state=%d) || "
 					   "(%d between t1.time_start "
 					   "and t1.time_end)))",
-					   JOB_RUNNING, start);
+					   base_state, start);
 			} else {
 				xstrfmtcat(*extra,
 					   "(t1.time_start && "
@@ -845,7 +844,8 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 
 	/* get the dimensions of this cluster so we know how to deal
 	   with the hostlists */
-	query = xstrdup_printf("select dimensions from %s where name='%s'",
+	query = xstrdup_printf("select dimensions, flags from %s where "
+			       "name='%s'",
 			       cluster_table,
 			       (char *)list_peek(job_cond->cluster_list));
 
@@ -862,7 +862,14 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 		      (char *)list_peek(job_cond->cluster_list));
 		return NULL;
 	}
-	dims = atoi(row[0]);
+
+	/* On a Cray System when dealing with hostlists as we are here
+	   this always needs to be 1.
+	*/
+	if (slurm_atoul(row[1]) & CLUSTER_FLAG_CRAYXT)
+		dims = 1;
+	else
+		dims = atoi(row[0]);
 
 	temp_hl = hostlist_create_dims(job_cond->used_nodes, dims);
 	if (hostlist_count(temp_hl) <= 0) {
@@ -918,6 +925,10 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 			if (local_cluster->end == 0) {
 				local_cluster->end = now;
 				(*curr_cluster) = local_cluster;
+			} else if (!(*curr_cluster)
+				   || (((local_cluster_t *)(*curr_cluster))->end
+				       < local_cluster->end)) {
+				(*curr_cluster) = local_cluster;
 			}
 		} else
 			_destroy_local_cluster(local_cluster);
@@ -978,16 +989,16 @@ extern int good_nodes_from_inx(List local_cluster_list,
 	return 1;
 }
 
-extern char *setup_job_cluster_cond_limits(mysql_conn_t *mysql_conn,
-					   slurmdb_job_cond_t *job_cond,
-					   char *cluster_name, char **extra)
+extern int setup_job_cluster_cond_limits(mysql_conn_t *mysql_conn,
+					 slurmdb_job_cond_t *job_cond,
+					 char *cluster_name, char **extra)
 {
 	int set = 0;
 	ListIterator itr = NULL;
 	char *object = NULL;
 
 	if (!job_cond)
-		return NULL;
+		return SLURM_SUCCESS;
 
 	/* this must be done before resvid_list since we set
 	   resvid_list up here */
@@ -1041,6 +1052,78 @@ no_resv:
 		xstrcat(*extra, ")");
 	}
 
+	if (job_cond->state_list && list_count(job_cond->state_list)) {
+		itr = list_iterator_create(job_cond->state_list);
+		while ((object = list_next(itr))) {
+			uint32_t state = (uint32_t)slurm_atoul(object);
+			state &= JOB_STATE_BASE;
+			if (state == JOB_SUSPENDED)
+				break;
+		}
+		list_iterator_destroy(itr);
+
+		if (object) {
+			MYSQL_RES *result = NULL;
+			MYSQL_ROW row;
+			char *query = xstrdup_printf(
+				"select job_db_inx from \"%s_%s\"",
+				cluster_name, suspend_table);
+			if (job_cond->usage_start) {
+				if (!job_cond->usage_end) {
+					xstrfmtcat(query,
+						   " where (!time_end "
+						   "|| (%d between "
+						   "time_start and time_end))",
+						   (int)job_cond->usage_start);
+				} else {
+					xstrfmtcat(query,
+						   " where (!time_end "
+						   "|| (time_start && "
+						   "((%d between time_start "
+						   "and time_end) "
+						   "|| (time_start between "
+						   "%d and %d))))",
+						   (int)job_cond->usage_start,
+						   (int)job_cond->usage_start,
+						   (int)job_cond->usage_end);
+				}
+			} else if (job_cond->usage_end) {
+				xstrfmtcat(query, " where (time_start && "
+					   "time_start < %d)",
+					   (int)job_cond->usage_end);
+			}
+
+			debug3("%d(%s:%d) query\n%s",
+			       mysql_conn->conn, THIS_FILE, __LINE__, query);
+			result = mysql_db_query_ret(mysql_conn, query, 0);
+			xfree(query);
+			if (!result)
+				return SLURM_ERROR;
+			set = 0;
+			while ((row = mysql_fetch_row(result))) {
+				if (set)
+					xstrfmtcat(*extra,
+						   " || t1.job_db_inx=%s",
+						   row[0]);
+				else {
+					set = 1;
+					if (*extra)
+						xstrfmtcat(
+							*extra,
+							" || (t1.job_db_inx=%s",
+							row[0]);
+					else
+						xstrfmtcat(*extra, " where "
+							   "(t1.job_db_inx=%s",
+							   row[0]);
+				}
+			}
+			mysql_free_result(result);
+			if (set)
+				xstrcat(*extra, ")");
+		}
+	}
+
 	return SLURM_SUCCESS;
 }
 
@@ -1125,6 +1208,24 @@ extern int setup_job_cond_limits(mysql_conn_t *mysql_conn,
 		xstrcat(*extra, ")");
 	}
 
+	if (job_cond->jobname_list && list_count(job_cond->jobname_list)) {
+		set = 0;
+		if (*extra)
+			xstrcat(*extra, " && (");
+		else
+			xstrcat(*extra, " where (");
+
+		itr = list_iterator_create(job_cond->jobname_list);
+		while ((object = list_next(itr))) {
+			if (set)
+				xstrcat(*extra, " || ");
+			xstrfmtcat(*extra, "t1.job_name='%s'", object);
+			set = 1;
+		}
+		list_iterator_destroy(itr);
+		xstrcat(*extra, ")");
+	}
+
 	if (job_cond->partition_list && list_count(job_cond->partition_list)) {
 		set = 0;
 		if (*extra)
@@ -1184,11 +1285,11 @@ extern int setup_job_cond_limits(mysql_conn_t *mysql_conn,
 			xstrcat(*extra, " where (");
 
 		if (job_cond->cpus_max) {
-			xstrfmtcat(*extra, "(t1.alloc_cpus between %u and %u))",
+			xstrfmtcat(*extra, "(t1.cpus_alloc between %u and %u))",
 				   job_cond->cpus_min, job_cond->cpus_max);
 
 		} else {
-			xstrfmtcat(*extra, "(t1.alloc_cpus='%u'))",
+			xstrfmtcat(*extra, "(t1.cpus_alloc='%u'))",
 				   job_cond->cpus_min);
 
 		}
@@ -1202,11 +1303,11 @@ extern int setup_job_cond_limits(mysql_conn_t *mysql_conn,
 
 		if (job_cond->nodes_max) {
 			xstrfmtcat(*extra,
-				   "(t1.alloc_nodes between %u and %u))",
+				   "(t1.nodes_alloc between %u and %u))",
 				   job_cond->nodes_min, job_cond->nodes_max);
 
 		} else {
-			xstrfmtcat(*extra, "(t1.alloc_nodes='%u'))",
+			xstrfmtcat(*extra, "(t1.nodes_alloc='%u'))",
 				   job_cond->nodes_min);
 
 		}
@@ -1241,6 +1342,7 @@ extern int setup_job_cond_limits(mysql_conn_t *mysql_conn,
 		while ((object = list_next(itr))) {
 			if (set)
 				xstrcat(*extra, " || ");
+
 			_state_time_string(extra, (uint32_t)slurm_atoul(object),
 					   job_cond->usage_start,
 					   job_cond->usage_end);
