@@ -47,6 +47,10 @@
 #  include <inttypes.h>
 #endif
 
+#if defined(__NetBSD__)
+#include <sys/types.h> /* for pid_t */
+#include <sys/signal.h> /* for SIGKILL */
+#endif
 #include <sys/types.h>
 
 #include "slurm/slurm.h"
