@@ -112,6 +112,7 @@ static slurm_select_ops_t *_other_select_get_ops(slurm_select_context_t *c)
 		"select_p_select_jobinfo_xstrdup",
 		"select_p_update_block",
 		"select_p_update_sub_node",
+		"select_p_fail_cnode",
 		"select_p_get_info_from_plugin",
 		"select_p_update_node_config",
 		"select_p_update_node_state",
@@ -750,6 +751,20 @@ extern int other_update_sub_node (update_block_msg_t *block_desc_ptr)
 		  update_sub_node))(block_desc_ptr);
 }
 
+/*
+ * Fail certain cnodes in a blocks midplane (usually comes from the
+ *        IBM runjob mux)
+ * IN step_ptr - step that failed
+ */
+extern int other_fail_cnode (struct step_record *step_ptr)
+{
+	if (other_select_init() < 0)
+		return SLURM_ERROR;
+
+	return (*(other_select_context->ops.
+		  fail_cnode))(step_ptr);
+}
+
 /*
  * Get select data from a plugin
  * IN dinfo     - type of data to get from the node record
