@@ -44,9 +44,9 @@ void invalidSwitchCombo(char *good, char *bad);
  */
 sacct_parameters_t params;
 print_field_t fields[] = {
-	{10, "AllocCPUS", print_fields_int, PRINT_ALLOC_CPUS},
+	{10, "AllocCPUS", print_fields_uint, PRINT_ALLOC_CPUS},
 	{10, "Account", print_fields_str, PRINT_ACCOUNT},
-	{7,  "AssocID", print_fields_int, PRINT_ASSOCID},
+	{7,  "AssocID", print_fields_uint, PRINT_ASSOCID},
 	{10, "AveCPU", print_fields_str, PRINT_AVECPU},
 	{10, "AvePages", print_fields_str, PRINT_AVEPAGES},
 	{10, "AveRSS", print_fields_str, PRINT_AVERSS},
@@ -61,35 +61,35 @@ print_field_t fields[] = {
 	{19, "Eligible", print_fields_date, PRINT_ELIGIBLE},
 	{19, "End", print_fields_date, PRINT_END},
 	{8,  "ExitCode", print_fields_str, PRINT_EXITCODE},
-	{6,  "GID", print_fields_int, PRINT_GID},
+	{6,  "GID", print_fields_uint, PRINT_GID},
 	{9,  "Group", print_fields_str, PRINT_GROUP},
 	{-12, "JobID", print_fields_str, PRINT_JOBID},
 	{10, "JobName", print_fields_str, PRINT_JOBNAME},
 	{9,  "Layout", print_fields_str, PRINT_LAYOUT},
 	{8,  "MaxPages", print_fields_str, PRINT_MAXPAGES},
 	{12, "MaxPagesNode", print_fields_str, PRINT_MAXPAGESNODE},
-	{14, "MaxPagesTask", print_fields_int, PRINT_MAXPAGESTASK},
+	{14, "MaxPagesTask", print_fields_uint, PRINT_MAXPAGESTASK},
 	{10, "MaxRSS", print_fields_str, PRINT_MAXRSS},
 	{10, "MaxRSSNode", print_fields_str, PRINT_MAXRSSNODE},
-	{10, "MaxRSSTask", print_fields_int, PRINT_MAXRSSTASK},
+	{10, "MaxRSSTask", print_fields_uint, PRINT_MAXRSSTASK},
 	{10, "MaxVMSize", print_fields_str, PRINT_MAXVSIZE},
 	{14, "MaxVMSizeNode", print_fields_str, PRINT_MAXVSIZENODE},
-	{14, "MaxVMSizeTask", print_fields_int, PRINT_MAXVSIZETASK},
+	{14, "MaxVMSizeTask", print_fields_uint, PRINT_MAXVSIZETASK},
 	{10, "MinCPU", print_fields_str, PRINT_MINCPU},
 	{10, "MinCPUNode", print_fields_str, PRINT_MINCPUNODE},
-	{10, "MinCPUTask", print_fields_int, PRINT_MINCPUTASK},
-	{10, "NCPUS", print_fields_int, PRINT_ALLOC_CPUS},
+	{10, "MinCPUTask", print_fields_uint, PRINT_MINCPUTASK},
+	{10, "NCPUS", print_fields_uint, PRINT_ALLOC_CPUS},
 	{8,  "NNodes", print_fields_str, PRINT_NNODES},
 	{15, "NodeList", print_fields_str, PRINT_NODELIST},
-	{8,  "NTasks", print_fields_int, PRINT_NTASKS},
-	{10, "Priority", print_fields_int, PRINT_PRIO},
+	{8,  "NTasks", print_fields_uint, PRINT_NTASKS},
+	{10, "Priority", print_fields_uint, PRINT_PRIO},
 	{10, "Partition", print_fields_str, PRINT_PARTITION},
 	{10, "QOS", print_fields_str, PRINT_QOS},
-	{6,  "QOSRAW", print_fields_int, PRINT_QOSRAW},
-	{8,  "ReqCPUS", print_fields_int, PRINT_REQ_CPUS},
+	{6,  "QOSRAW", print_fields_uint, PRINT_QOSRAW},
+	{8,  "ReqCPUS", print_fields_uint, PRINT_REQ_CPUS},
 	{10, "Reserved", print_fields_time_from_secs, PRINT_RESV},
 	{10, "ResvCPU", print_fields_time_from_secs, PRINT_RESV_CPU},
-	{10, "ResvCPURAW", print_fields_int, PRINT_RESV_CPU},
+	{10, "ResvCPURAW", print_fields_uint, PRINT_RESV_CPU},
 	{19, "Start", print_fields_date, PRINT_START},
 	{10, "State", print_fields_str, PRINT_STATE},
 	{19, "Submit", print_fields_date, PRINT_SUBMIT},
@@ -97,11 +97,11 @@ print_field_t fields[] = {
 	{10, "SystemCPU", print_fields_str, PRINT_SYSTEMCPU},
 	{10, "Timelimit", print_fields_str, PRINT_TIMELIMIT},
 	{10, "TotalCPU", print_fields_str, PRINT_TOTALCPU},
-	{6,  "UID", print_fields_int, PRINT_UID},
+	{6,  "UID", print_fields_uint, PRINT_UID},
 	{9,  "User", print_fields_str, PRINT_USER},
 	{10, "UserCPU", print_fields_str, PRINT_USERCPU},
 	{10, "WCKey", print_fields_str, PRINT_WCKEY},
-	{10, "WCKeyID", print_fields_int, PRINT_WCKEYID},
+	{10, "WCKeyID", print_fields_uint, PRINT_WCKEYID},
 	{0,  NULL, NULL, 0}};
 
 List jobs = NULL;
