@@ -84,8 +84,11 @@ _scontrol_load_jobs(job_info_msg_t ** job_buffer_pptr, uint32_t job_id)
 	if (all_flag)
 		show_flags |= SHOW_ALL;
 
-	if (detail_flag)
+	if (detail_flag) {
 		show_flags |= SHOW_DETAIL;
+		if (detail_flag > 1)
+			show_flags |= SHOW_DETAIL2;
+	}
 
 	if (old_job_info_ptr) {
 		if (last_show_flags != show_flags)
