@@ -85,7 +85,7 @@ _build_path(char* fname)
 	}
 
 	/* check if already absolute path */
-	if (file_name[0] == '/')
+	if ((file_name[0] == '/') || (file_name[0] == '.'))
 		return file_name;
 
 	/* search for the file using PATH environment variable */
@@ -113,9 +113,13 @@ _build_path(char* fname)
 static void
 _set_range(int low_num, int high_num, char *exec_name, bool ignore_duplicates)
 {
+#if defined HAVE_BG_FILES && !defined HAVE_BG_L_P
+	/* Use symbols from the runjob.so library provided by IBM.
+	 * Do NOT use debugger symbols local to the srun command */
+#else
 	int i;
 
-	for (i=low_num; i<=high_num; i++) {
+	for (i = low_num; i <= high_num; i++) {
 		MPIR_PROCDESC *tv;
 		tv = &MPIR_proctable[i];
 		if (tv->executable_name == NULL) {
@@ -125,6 +129,7 @@ _set_range(int low_num, int high_num, char *exec_name, bool ignore_duplicates)
 			      i);
 		}
 	}
+#endif
 }
 
 static void
@@ -178,13 +183,19 @@ mpir_set_multi_name(int ntasks, const char *config_fname)
 	FILE *config_fd;
 	char line[256];
 	char *ranks, *exec_name, *p, *ptrptr;
-	int line_num = 0, i;
+	int line_num = 0;
 
-	for (i=0; i<ntasks; i++) {
+#if defined HAVE_BG_FILES && !defined HAVE_BG_L_P
+	/* Use symbols from the runjob.so library provided by IBM.
+	 * Do NOT use debugger symbols local to the srun command */
+#else
+	int i;
+	for (i = 0; i < ntasks; i++) {
 		MPIR_PROCDESC *tv;
 		tv = &MPIR_proctable[i];
 		tv->executable_name = NULL;
 	}
+#endif
 
 	config_fd = fopen(config_fname, "r");
 	if (config_fd == NULL) {
@@ -226,17 +237,26 @@ mpir_set_multi_name(int ntasks, const char *config_fname)
 extern void
 mpir_init(int num_tasks)
 {
+#if defined HAVE_BG_FILES && !defined HAVE_BG_L_P
+	/* Use symbols from the runjob.so library provided by IBM.
+	 * Do NOT use debugger symbols local to the srun command */
+#else
 	MPIR_proctable_size = num_tasks;
 	MPIR_proctable = xmalloc(sizeof(MPIR_PROCDESC) * num_tasks);
 	if (MPIR_proctable == NULL) {
 		error("Unable to initialize MPIR_proctable: %m");
 		exit(error_exit);
 	}
+#endif
 }
 
 extern void
 mpir_cleanup(void)
 {
+#if defined HAVE_BG_FILES && !defined HAVE_BG_L_P
+	/* Use symbols from the runjob.so library provided by IBM.
+	 * Do NOT use debugger symbols local to the srun command */
+#else
 	int i;
 
 	for (i = 0; i < MPIR_proctable_size; i++) {
@@ -244,11 +264,16 @@ mpir_cleanup(void)
 		xfree(MPIR_proctable[i].executable_name);
 	}
 	xfree(MPIR_proctable);
+#endif
 }
 
 extern void
 mpir_set_executable_names(const char *executable_name)
 {
+#if defined HAVE_BG_FILES && !defined HAVE_BG_L_P
+	/* Use symbols from the runjob.so library provided by IBM.
+	 * Do NOT use debugger symbols local to the srun command */
+#else
 	int i;
 
 	for (i = 0; i < MPIR_proctable_size; i++) {
@@ -259,11 +284,16 @@ mpir_set_executable_names(const char *executable_name)
 			exit(error_exit);
 		}
 	}
+#endif
 }
 
 extern void
-mpir_dump_proctable()
+mpir_dump_proctable(void)
 {
+#if defined HAVE_BG_FILES && !defined HAVE_BG_L_P
+	/* Use symbols from the runjob.so library provided by IBM.
+	 * Do NOT use debugger symbols local to the srun command */
+#else
 	MPIR_PROCDESC *tv;
 	int i;
 
@@ -274,6 +304,7 @@ mpir_dump_proctable()
 		info("task:%d, host:%s, pid:%d, executable:%s",
 		     i, tv->host_name, tv->pid, tv->executable_name);
 	}
+#endif
 }
 
 static int
