@@ -58,18 +58,18 @@ print_field_t fields[] = {
 	{-12, "JobID", print_fields_str, PRINT_JOBID},
 	{8, "MaxPages", print_fields_str, PRINT_MAXPAGES},
 	{12, "MaxPagesNode", print_fields_str, PRINT_MAXPAGESNODE},
-	{14, "MaxPagesTask", print_fields_int, PRINT_MAXPAGESTASK},
+	{14, "MaxPagesTask", print_fields_uint, PRINT_MAXPAGESTASK},
 	{10, "MaxRSS", print_fields_str, PRINT_MAXRSS},
 	{10, "MaxRSSNode", print_fields_str, PRINT_MAXRSSNODE},
-	{10, "MaxRSSTask", print_fields_int, PRINT_MAXRSSTASK},
+	{10, "MaxRSSTask", print_fields_uint, PRINT_MAXRSSTASK},
 	{10, "MaxVMSize", print_fields_str, PRINT_MAXVSIZE},
 	{14, "MaxVMSizeNode", print_fields_str, PRINT_MAXVSIZENODE},
-	{14, "MaxVMSizeTask", print_fields_int, PRINT_MAXVSIZETASK},
+	{14, "MaxVMSizeTask", print_fields_uint, PRINT_MAXVSIZETASK},
 	{10, "MinCPU", print_fields_str, PRINT_MINCPU},
 	{10, "MinCPUNode", print_fields_str, PRINT_MINCPUNODE},
-	{10, "MinCPUTask", print_fields_int, PRINT_MINCPUTASK},
+	{10, "MinCPUTask", print_fields_uint, PRINT_MINCPUTASK},
 	{20, "Nodelist", print_fields_str, PRINT_NODELIST},
-	{8, "NTasks", print_fields_int, PRINT_NTASKS},
+	{8, "NTasks", print_fields_uint, PRINT_NTASKS},
 	{20, "Pids", print_fields_str, PRINT_PIDS},
 	{0, NULL, NULL, 0}};
 
