@@ -73,6 +73,7 @@
 #define OPT_LONG_USER      0x105
 #define OPT_LONG_BLOCK_ERR 0x106
 #define OPT_LONG_FRONT_END 0x107
+#define OPT_LONG_FLAGS     0x108
 
 /* getopt_long options, integers but not characters */
 
@@ -127,6 +128,7 @@ extern void parse_command_line(int argc, char *argv[])
 		{"version",                             no_argument, 0, 'V'},
 		{"block_err", no_argument,       0, OPT_LONG_BLOCK_ERR},
 		{"clear",     no_argument,       0, OPT_LONG_CLEAR},
+		{"flags",     required_argument, 0, OPT_LONG_FLAGS},
 		{"front_end", no_argument,       0, OPT_LONG_FRONT_END},
 		{"get",       no_argument,       0, OPT_LONG_GET},
 		{"help",      no_argument,       0, OPT_LONG_HELP},
@@ -268,6 +270,14 @@ extern void parse_command_line(int argc, char *argv[])
 		case (int) OPT_LONG_CLEAR:
 			params.mode_clear = true;
 			break;
+		case (int) OPT_LONG_FLAGS:
+			if (!strncasecmp(optarg, "perm", 4))
+				params.flags = TRIGGER_FLAG_PERM;
+			else {
+				error("Invalid flags %s", optarg);
+				exit(1);
+			}
+			break;
 		case (int) OPT_LONG_FRONT_END:
 			params.front_end = true;
 			break;
@@ -307,6 +317,7 @@ static void _init_options( void )
 	params.bu_ctld_fail = false;
 	params.bu_ctld_res_op = false;
 	params.bu_ctld_as_ctrl = false;
+	params.flags        = 0;
 	params.front_end    = false;
 	params.node_down    = false;
 	params.node_drained = false;
@@ -326,7 +337,7 @@ static void _init_options( void )
 	params.reconfig     = false;
 	params.time_limit   = false;
 	params.node_up      = false;
-	params.user_id      = 0;
+	params.user_id      = NO_VAL;
 	params.verbose      = 0;
 }
 
@@ -338,6 +349,7 @@ static void _print_options( void )
 	verbose("get          = %s", params.mode_get ? "true" : "false");
 	verbose("clear        = %s", params.mode_clear ? "true" : "false");
 	verbose("block_err    = %s", params.block_err ? "true" : "false");
+	verbose("flags        = %u", params.flags);
 	verbose("front_end    = %s", params.front_end ? "true" : "false");
 	verbose("job_id       = %u", params.job_id);
 	verbose("job_fini     = %s", params.job_fini ? "true" : "false");
@@ -353,7 +365,10 @@ static void _print_options( void )
 	verbose("reconfig     = %s", params.reconfig ? "true" : "false");
 	verbose("time_limit   = %s", params.time_limit ? "true" : "false");
 	verbose("trigger_id   = %u", params.trigger_id);
-	verbose("user_id      = %u", params.user_id);
+	if (params.user_id == NO_VAL)
+		verbose("user_id      = N/A");
+	else
+		verbose("user_id      = %u", params.user_id);
 	verbose("verbose      = %d", params.verbose);
 	verbose("primary_slurmctld_failure            = %s",
 		params.pri_ctld_fail ? "true" : "false");
@@ -388,8 +403,8 @@ static void _validate_options( void )
 		exit(1);
 	}
 
-	if (params.mode_clear
-	&&  ((params.trigger_id + params.job_id + params.user_id) == 0)) {
+	if (params.mode_clear && (params.user_id == NO_VAL) &&
+	    (params.trigger_id == 0) && (params.job_id == 0)) {
 		error("You must specify a --id, --jobid, or --user to clear");
 		exit(1);
 	}
@@ -489,6 +504,7 @@ Usage: strigger [--set | --get | --clear] [OPTIONS]\n\
                       trigger event when primary slurmctld acct buffer full\n\
   -F, --fail          trigger event when node is expected to FAIL\n\
   -f, --fini          trigger event when job finishes\n\
+      --flags=perm    trigger event flag (perm = permanent)\n\n\
   -g, --primary_slurmdbd_failure\n\
                       trigger when primary slurmdbd fails\n\
   -G, --primary_slurmdbd_resumed_operation\n\
